package com.frk;

import soot.*;
import soot.jimple.*;
import soot.jimple.internal.JIfStmt;
import soot.options.Options;
import soot.toolkits.graph.*;
import soot.toolkits.scalar.*;

import java.util.*;

public class AssertDependencyAnalyzer {
    public static void main(String[] args) {
        // 设置Soot环境
        Options.v().set_prepend_classpath(true);
        Options.v().set_whole_program(true);
        Options.v().set_app(true);
        Options.v().set_src_prec(Options.src_prec_java);
        Options.v().set_process_dir(Collections.singletonList("D:\\learn\\junit-test\\test\\solve2\\EightQueens_1666011085487\\src\\test\\java\\net\\mooctest\\EightQueensTest.java"));
//        "JavaParser"
        // 设置主类（Main class）
        Options.v().set_main_class("D:\\learn\\junit-test\\openclover\\src\\main\\java\\com\\frk\\AssertDependencyAnalyzer.java"); // 这里需要替换为你的主类名

        // 设置入口点
        List<SootMethod> entryPoints = new ArrayList<>();
//        SootClass mainClass = Scene.v().loadClassAndSupport("D:\\learn\\junit-test\\openclover\\src\\main\\java\\com\\frk\\AssertDependencyAnalyzer.java");
//        mainClass.setApplicationClass();
//        SootMethod mainMethod = mainClass.getMethodByName("main");
//        entryPoints.add(mainMethod);
//        Scene.v().setEntryPoints(entryPoints);
        // 注册一个Transform
        PackManager.v().getPack("jtp").add(new Transform("jtp.myTransform", new BodyTransformer() {
            @Override
            protected void internalTransform(Body body, String phase, Map<String, String> options) {
                analyzeMethod(body);
            }
        }));

        // 启动Soot
        System.out.println("Before launching Soot");

        soot.Main.main(args);

        System.out.println("Exiting main method");
    }

    private static void analyzeMethod(Body body) {
        // 构建控制流图
        UnitGraph cfg = new BriefUnitGraph(body);

        // 查找assert语句
        for (Unit unit : body.getUnits()) {
            if (unit instanceof JIfStmt) {
                JIfStmt ifStmt = (JIfStmt) unit;
                if (isAssertStatement(ifStmt)) {
                    // 分析assert语句的依赖
                    analyzeDependencies(ifStmt, cfg);
                }
            }
        }
        System.out.println("Exiting analyzeMethod");
    }

    private static boolean isAssertStatement(JIfStmt ifStmt) {
        // 假设assert语句是if语句中的条件为assert表达式的情况
        // 可以根据实际情况调整判定逻辑
        Value condition = ifStmt.getCondition();
        return condition.toString().contains("assert");
    }

    private static void analyzeDependencies(Unit assertUnit, UnitGraph cfg) {
        // 使用反向数据流分析
        SimpleLocalDefs localDefs = new SimpleLocalDefs(cfg);
        SimpleLiveLocals liveLocals = new SimpleLiveLocals(cfg);

        for (Unit pred : cfg.getPredsOf(assertUnit)) {
            List<Unit> defs = localDefs.getDefsOfAt((Local) ((JIfStmt) assertUnit).getCondition(), pred);
            System.out.println("Assert statement at " + assertUnit + " depends on " + defs);
        }

        // 打印所有影响assert语句的语句
        List<Unit> reachableUnits = getReachableUnits(assertUnit, cfg);
        for (Unit unit : reachableUnits) {
            System.out.println("Unit " + unit + " affects the assert statement.");
        }
    }

    private static List<Unit> getReachableUnits(Unit target, UnitGraph cfg) {
        List<Unit> reachableUnits = new ArrayList<>();
        Queue<Unit> queue = new LinkedList<>();
        Set<Unit> visited = new HashSet<>();

        queue.add(target);
        visited.add(target);

        while (!queue.isEmpty()) {
            Unit current = queue.poll();
            reachableUnits.add(current);

            for (Unit pred : cfg.getPredsOf(current)) {
                if (!visited.contains(pred)) {
                    visited.add(pred);
                    queue.add(pred);
                }
            }
        }

        return reachableUnits;
    }
}

