package com.frk.mycode;

import com.atlassian.clover.CloverDatabase;
import com.atlassian.clover.CodeType;
import com.atlassian.clover.CoverageData;
import com.atlassian.clover.CoverageDataSpec;
import com.atlassian.clover.api.registry.*;
import com.atlassian.clover.registry.entities.FullBranchInfo;
import com.atlassian.clover.registry.entities.FullProjectInfo;
import com.atlassian.clover.registry.entities.TestCaseInfo;

import java.io.PrintStream;
import java.util.BitSet;
import java.util.Set;

/**
 * @date 2024/4/12
 */
public class DBRead {
    public static void main(String[] args) throws Exception {
//        String s = "D:\\learn\\junit-test\\test\\solve\\EightQueens_1664963256377\\target\\clover\\clover.db";
        String s = "D:\\learn\\junit-test\\openclover\\target\\clover\\clover.db";
        // read clover database together with coverage recording files, use time span=0 (latest build)
        CloverDatabase db = CloverDatabase.loadWithCoverage(s, new CoverageDataSpec());
        ProjectInfo projectInfo = db.getRegistry().getProject();
        FullProjectInfo model = db.getModel(CodeType.ALL);
        // print some project details
        CoverageData cd= db.getCoverageData();
        BitSet allHits = cd.getAllHits();
        printProject(projectInfo, System.out, cd);


    }

    private static void printProject(ProjectInfo db, PrintStream out, CoverageData cd) {
        for (PackageInfo packageInfo : db.getAllPackages()) {
            out.println("package: " + packageInfo.getName());
            for (FileInfo fileInfo : packageInfo.getFiles()) {
                out.println("\tfile: " + fileInfo.getName());
                for (ClassInfo classInfo : fileInfo.getClasses()) {
                    out.println("\t\tclass: " + classInfo.getName());
                    for (MethodInfo methodInfo : classInfo.getMethods()) {
//                        int dataIndex = methodInfo.getDataIndex();
//                        int dataLength = methodInfo.getDataLength();
//                        BitSet methodCoverage = allHits.get(dataIndex, dataIndex + dataLength);
                        System.out.println("\t\t\tMethod " + methodInfo.getName());
                        for (BranchInfo branchInfo : methodInfo.getBranches()) {
                            System.out.println("\t\t\t\tbrach: " + branchInfo.getContext());
                            int dataIndex = branchInfo.getDataIndex();
                            int dataLength = branchInfo.getDataLength();
                            for (TestCaseInfo test : cd.getTests()) {
                                // hitsFor表示 该测试方法命中了哪些 数据(语句、分支、方法)
                                BitSet hitsFor = cd.getHitsFor(test);
                                // 检查测试方法是否覆盖了这个分支
                                BitSet branchHits = hitsFor.get(dataIndex, dataIndex + dataLength);
                                if (!branchHits.isEmpty()) {
                                    // System.out.println(hitsFor);
                                    System.out.println("\t\t\t\t\tTest方法: " + test.getTestName() + " covers branch at " + branchInfo.getStartLine());
                                }
                            }
//                            int startLine = branchInfo.getStartLine();
//                            System.out.println("\t\t\t\t\tbranch Line: " + startLine);
//                            FullBranchInfo fullBranchInfo = (FullBranchInfo) branchInfo;
//                            Set<TestCaseInfo> testsCovering = cd.getTestsCovering(fullBranchInfo);
//                            System.out.println("\t\t\t\t\tcoverage By: " + testsCovering);

                        }
                    }

                }
            }
        }
    }
}
