package com.frk.mycode;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.atlassian.clover.registry.entities.TestCaseInfo;
import com.frk.mycode.branch.BranchResult;
import com.frk.mycode.mutation.Mutation;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

/**
 * @date 2024/4/12
 */
public class DecreaseTestCase {
    public static void main(String[] args) throws InterruptedException {
        /**
         * 跑一个试一下
         */
        if (false) {
            testone();
            return;
        }
        /**
         * 项目一级路径，下一级目录应该是很多个子项目例如EightQueens_XXX
         */
        String sourcePath = "D:\\learn\\junit-test\\test\\EightQueens_N";
        File sourceDir = new File(sourcePath);
//        replaceAll(sourceDir);
        int success = 0;
        List<String> error = new ArrayList<>();
        for (File tempFile : sourceDir.listFiles()) {
            String path = tempFile.getPath();
            if (tempFile.isDirectory()) {
                        System.out.println("RUN " + path);
                    if (!OpencloverRunUtil.FAILURE.equals(OpencloverRunUtil.execCMD(path))) {
                        List<BranchResult> results = ReadCloverDB.readDB(path);
                        if (results != null) {
                            generateResultFile(results, tempFile.getAbsolutePath());
                        }
                    } else  {
                        error.add(path);
                    }
            }
        }
        /**
         * 分支覆盖统计
         */
        ReadCloverDB.printDecreaseInfo();
        /**
         * 打印编译不通过的信息
         */
        for (String s : error) {
            System.out.println(s);
        }
        System.out.println(1.0 * success / sourceDir.listFiles().length);
    }
    public static void testone() {
        String sourcePath = "D:\\learn\\junit-test\\test\\origin\\EightQueens_1665842723113";

        File sourceDir = new File(sourcePath);
        if (!OpencloverRunUtil.FAILURE.equals(OpencloverRunUtil.execCMD(sourcePath))) {
            List<BranchResult> results = ReadCloverDB.readDB(sourcePath);
            if (results != null) {
                generateResultFile(results, sourceDir.getAbsolutePath());
            }
        } else  {
        }
    }
    private static void generateResultFile(List<BranchResult> results, String absolutePath) {
        JSONObject jsonObject = new JSONObject();
        String jsonTarget = absolutePath + "//target//" + "myresult.json";
        jsonObject.put("file", absolutePath);
        JSONArray jsonArray = new JSONArray();
        for (BranchResult branchResult : results) {
            JSONObject tmp = new JSONObject();
            tmp.put("branch", branchResult.getFilePath());
            tmp.put("line", branchResult.getBranchStartLine());
            tmp.put("isCover", branchResult.isCovered());
            JSONArray coverArray = new JSONArray();
            if (!Objects.isNull(branchResult.getTestCaseInfo()))
                for (TestCaseInfo testCaseInfo : branchResult.getTestCaseInfo()) {
                    coverArray.add(testCaseInfo.getClassName() + ":" + testCaseInfo.getTestName());
                }
            tmp.put("testCase", coverArray);
            jsonArray.add(tmp);
        }
        jsonObject.put("coverMessage", jsonArray);
        writeJSONString(jsonObject.toJSONString(), jsonTarget);

    }
    public static void writeJSONString(String jsonString, String outputFileName){
        try {
            BufferedWriter output = new BufferedWriter(new FileWriter(outputFileName));
            output.write(jsonString);
            output.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public static void  replaceAll(File sourceDir) {
        File[] subProjects = sourceDir.listFiles(File::isDirectory);
        String targetPath = "src\\main\\java\\net\\mooctest";
        if (subProjects != null && subProjects.length > 0) {
            // 获取第一个子项目的 "src\\main\\java\\net\\mooctest" 目录下的所有 Java 文件
            File firstSubProject = subProjects[0];
            File[] sourceFiles = getJavaFilesInDirectory(new File(firstSubProject, targetPath));

            if (sourceFiles != null && sourceFiles.length > 0) {
                // 遍历所有其他子项目
                for (int i = 1; i < subProjects.length; i++) {
                    // 替换每个子项目的 "src\\main\\java\\net\\mooctest" 目录下的 Java 文件
                    replaceJavaFiles(subProjects[i].getPath(), sourceFiles);
                }
            } else {
                System.out.println("未找到任何Java文件。");
            }
        } else {
            System.out.println("未找到任何子项目。");
            return;
        }
    }
    public static File[] getJavaFilesInDirectory(File directory) {
        if (!directory.exists() || !directory.isDirectory()) {
            return null;
        }
        return directory.listFiles((dir, name) -> name.endsWith(".java"));
    }

    // 将目标目录下的所有 Java 文件替换为源文件
    public static void replaceJavaFiles(String targetPath, File[] sourceFiles) {
        for (File sourceFile : sourceFiles) {
            String targetFilePath = targetPath + File.separator + "src\\main\\java\\net\\mooctest" + File.separator + sourceFile.getName();
            Path target = Paths.get(targetFilePath);
            if (Files.exists(target)) {
                try {
                    Files.copy(sourceFile.toPath(), target, java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println("替换文件：" + targetFilePath);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else {
                System.out.println("未找到目标文件：" + targetFilePath);
            }
        }
    }

}
