package com.frk.mycode;

import com.alibaba.fastjson.JSONObject;
import com.frk.mycode.mutation.Mutation;
import com.frk.mycode.mutation.MyTestCase;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

/**
 * @date 2024/4/28
 */
public class PITestUtil {

    public static List<Mutation> runAndParsePITest(String projectPath) {
        String result = execCMD(projectPath);
        if (result.equals("FAILURE")) {
            return null;
        }
        String pitestPath = projectPath + File.separator + "target" + File.separator + "pit-reports";
        File file = new File(pitestPath);
        if (!file.exists()) {
            return null;
        }
        ArrayList<Mutation> resultList = new ArrayList<>();
        for (File singlePackageFile : file.listFiles(File::isDirectory)) {
//            System.out.println(singlePackageFile);
            List<Mutation> mutationList = parseSinglePackageXML(singlePackageFile.getAbsolutePath());
            resultList.addAll(mutationList);
        }
        int killedCount = 0;
        for (Mutation mutation : resultList) {
            if (!mutation.getKilledByTestCases().isEmpty()) {
                killedCount++;
            }
        }
        System.out.println("Project: " + projectPath + "Total Mutation: " + resultList.size() + " Killed Mutation: " + killedCount + " percent: " + killedCount*100.0 / resultList.size() );
        String jsonString = JSONObject.toJSONString(resultList);
        File writeFile = new File(file.getAbsolutePath() + File.separator + "result.json");
        try(Writer write = new OutputStreamWriter(new FileOutputStream(writeFile), StandardCharsets.UTF_8)) {
            write.write(jsonString);
            write.flush();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return resultList;
    }
    public static String execCMD(String filepath) {
        String PITestRun = "";
        try {
            BufferedReader bufferedReader = null;
            Process process = null;
            PITestRun = "cmd /c mvn -file " + filepath + " clean test-compile org.pitest:pitest-maven:mutationCoverage";
            process = Runtime.getRuntime().exec(PITestRun);
            bufferedReader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            String line = null;
            while ((line = bufferedReader.readLine()) != null) {
                if (line.contains("BUILD FAILURE")) {
                    PITestRun = "FAILURE";
                }
//                System.out.println(line);
            }
            bufferedReader.close();
            process.destroy();
            process = null;
        } catch (IOException e) {
            // e.printStackTrace();
            PITestRun = "FAILURE";
        }
        return PITestRun;
    }

    /**
     *
     * @param xmlDirPath /项目/target/pit-reports/包的路径
     * @return
     * @throws IOException
     */
    public static List<Mutation> parseSinglePackageXML(String xmlDirPath) {
        ArrayList<Mutation> mutations = new ArrayList<>();
        File file = new File(xmlDirPath);
        for (File listFile : file.listFiles()) {
            if (listFile.isFile()) {
                if (listFile.getName().endsWith(".java.html")) {
                    List<Mutation> mutations1 = PITestUtil.parseXML(listFile.getAbsolutePath());
                    mutations.addAll(mutations1);
                }
            } else if (listFile.isDirectory()) {
                mutations.addAll(parseSinglePackageXML(listFile.getAbsolutePath()));
            }

        }
//        System.out.println(mutations);
        return mutations;
    }
    public static List<Mutation> parseXML(String xmlPath) {
        File file = new File(xmlPath);
        Document doc = null;
        try {
            doc = Jsoup.parse(file, "utf-8");
        } catch (IOException e) {
            e.printStackTrace();
        }
        Elements testMethods = doc.select("ul li");
        HashSet<MyTestCase> myTestCaseHashSet = new HashSet<>();
        for (Element testMethod : testMethods) {
            String methodName = testMethod.text();
            if (!methodName.contains("("))continue;
            String testMethodPath = methodName.substring(0, methodName.indexOf("("));
            int index = testMethodPath.lastIndexOf(".");
            MyTestCase myTestCase = new MyTestCase(xmlPath, testMethodPath.substring(0, index), testMethodPath.substring(index+1));
            myTestCaseHashSet.add(myTestCase);
//            System.out.println(myTestCase);
        }
//        System.out.println(myTestCaseHashSet);
        PITesrDecrease.allTestCases.addAll(myTestCaseHashSet);
        Elements mutations = doc.select("tr");
        List<Mutation> mutationList = new ArrayList<>();
        for (Element mutation : mutations) {
            Elements mutationInfos = mutation.select("p");
            if (mutationInfos.isEmpty()) {
                continue;
            }
            Elements aTags = mutation.select("td a");
            if (!aTags.isEmpty()) {
                String href = aTags.first().attr("href");
                String id = href.substring(href.lastIndexOf('_') + 1);
//                System.out.println("Mutation ID: " + id);
                for (Element mutationInfo : mutationInfos) {
//                    System.out.println("Mutation Info: " + mutationInfo.text());
                    Elements groudId = mutationInfo.select("span.pop");

                    String infoText = mutationInfo.text();
//                    System.out.println(groudId.text());
                    int endIndex = groudId.text().indexOf('.');
                    String type = groudId.text().substring(0, endIndex);
                    String location = infoText.split(" ")[3];
                    List<MyTestCase> killedByTestCases = new ArrayList<>();
                    int end = infoText.indexOf("→");
                    int begin = infoText.lastIndexOf(":");
                    String mutationDeTail = infoText.substring(begin+1, end).trim();
//                    type += " " + mutationDeTail;
                    String mutationResult = infoText.contains("→") ? infoText.split("→")[1].trim() : "none";
                    MyTestCase tesCase = null;
                    Mutation mutationObj = new Mutation(file.getParentFile().getName(), Integer.parseInt(id), type, location, file.getName().substring(0, file.getName().lastIndexOf(".html")),killedByTestCases);
                    if ((tesCase = findTestCases(myTestCaseHashSet, mutationInfo.text())) != null) {
                        killedByTestCases.add(tesCase);
                    }
                    mutationList.add(mutationObj);
                }
            }
        }
        return mutationList;
    }

    private static MyTestCase findTestCases(HashSet<MyTestCase> myTestCaseHashSet, String text) {
        for (MyTestCase myTestCase : myTestCaseHashSet) {
            String s = myTestCase.getPackagePath() + "." + myTestCase.getMethodName();
            if (text.contains(s)) {
                return myTestCase;
            }
        }
        return null;
    }

    public static void main(String[] args) throws IOException {
//        String s = parseXML("D:\\learn\\junit-test\\openclover\\src\\main\\resources\\IntNumber.java.html").toString();
//        System.out.println(s);
        String s = "D:\\learn\\junit-test\\openclover";
        runAndParsePITest(s);

    }
}
