package com.frk.mycode;

import com.atlassian.clover.CloverDatabase;
import com.atlassian.clover.CoverageData;
import com.atlassian.clover.CoverageDataSpec;
import com.atlassian.clover.api.registry.*;
import com.atlassian.clover.registry.entities.FullFileInfo;
import com.atlassian.clover.registry.entities.TestCaseInfo;
import com.frk.mycode.branch.BranchResult;
import com.frk.mycode.branch.MyBranchInfo;

import java.io.*;
import java.util.*;

/**
 * @date 2024/4/12
 */
public class ReadCloverDB {
    static int totalCount;
    /**
     * 被覆盖的分支-对应的覆盖该分支的测试用例
     */
    static HashMap<MyBranchInfo, TestCaseInfo> branchInfoTestCaseInfoHashMap = new HashMap<>();
    /**
     * 所有的分支信息
     */
    static LinkedHashSet<MyBranchInfo> allBranchInfo = new LinkedHashSet<>();
    /**
     * 已经存储在约简库中的测试用例
     */
    static HashSet<TestCaseInfo> existTestCase = new HashSet<>();

    /**
     * 已经存储在约简库中的测试用例
     */
    static List<TestCaseInfo> allTestCases = new ArrayList<>();

    static ArrayList<BranchResult> nowBranToTestCase;
    public static void main(String[] args) throws Exception {
        String s = "D:\\learn\\junit-test\\test\\solve\\EightQueens_1664963256377";
        readDB(s);
        System.out.println("总分支数量 " + allBranchInfo.size());
        System.out.println("约简后的测试用例数量 " + existTestCase.size());
        System.out.println("约简后的分支覆盖率 " + branchInfoTestCaseInfoHashMap.size()*1.0/allBranchInfo.size() * 100);
    }

    public static List<BranchResult> readDB(String s) {
        String path = s + "\\target\\clover\\clover.db";
        File file = new File(path);
        if (!file.exists())
            return null;
        try {
            nowBranToTestCase = new ArrayList<>();
            CloverDatabase db = CloverDatabase.loadWithCoverage(path, new CoverageDataSpec());
            ProjectInfo projectInfo = db.getRegistry().getProject();
            CoverageData cd = db.getCoverageData();
            allTestCases.addAll(cd.getTests());
            parseProject(projectInfo, System.out, cd, path);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return nowBranToTestCase;
    }
    public static void printDecreaseInfo() {
        System.out.println("===============================RESULT SHOW================================");
        System.out.println("总分支数量： " + allBranchInfo.size());
        System.out.println("约简前的测试用例数量： " + allTestCases.size());
        System.out.println("约简后的测试用例数量： " + existTestCase.size());
        System.out.println("约简后的分支覆盖率： " + branchInfoTestCaseInfoHashMap.size()*1.0/allBranchInfo.size() * 100.0);
        System.out.println("===============================ALL Branch================================");
        for (MyBranchInfo myBranchInfo : allBranchInfo) {
            System.out.println(myBranchInfo);
        }
        System.out.println("===============================ALL Selected Test Case================================");
        for (TestCaseInfo testCaseInfo: existTestCase) {
            System.out.println(testCaseInfo.getTestName());
        }
    }

    private static void parseProject(ProjectInfo projectInfo, PrintStream out, CoverageData cd, String path) {
        for (PackageInfo packageInfo : projectInfo.getAllPackages()) {
//            out.println("package: " + packageInfo.getName());
            for (FileInfo fileInfo : packageInfo.getFiles()) {
//                out.println("\tfile: " + fileInfo.getName());
                if (fileInfo.getName().endsWith("Test.java"))continue;
                for (ClassInfo classInfo : fileInfo.getClasses()) {
//                    out.println("\t\tclass: " + classInfo.getName());
                    for (MethodInfo methodInfo : classInfo.getMethods()) {
//                        System.out.println("\t\t\tMethod " + methodInfo.getName());
                        for (BranchInfo branchInfo : methodInfo.getBranches()) {
                            totalCount++;
//                            System.out.println("\t\t\t\tbranch: " + branchInfo.getContext());
                            int dataIndex = branchInfo.getDataIndex();
                            int dataLength = branchInfo.getDataLength();
                            MyBranchInfo myBranchInfo = new MyBranchInfo(packageInfo, fileInfo, methodInfo, branchInfo, path);
                            // 1.查看在不在所有的分支里面
                            if (allBranchInfo.contains(myBranchInfo)) {
//                                System.out.println(branchInfo + " " + branchInfo.getStartLine());
//                                System.out.println(myBranchInfo);
                            } else {
                                allBranchInfo.add(myBranchInfo);
                            }

                            // 2. 统计每个项目的覆盖信息
                            BranchResult branchResult = new BranchResult(fileInfo.getName(), branchInfo.getStartLine(), false);
                            nowBranToTestCase.add(branchResult);
                            for (TestCaseInfo test : cd.getTests()) {
                                // hitsFor表示 该测试方法命中了哪些 数据(语句、分支、方法)
                                BitSet hitsFor = cd.getHitsFor(test);
                                // 检查测试方法是否覆盖了这个分支
                                BitSet branchHits = hitsFor.get(dataIndex, dataIndex + dataLength);
                                if (!branchHits.isEmpty()) {
                                    branchResult.setCovered(true);
                                    if (null == branchResult.getTestCaseInfo())
                                        branchResult.setTestCaseInfo(new ArrayList<>());
                                    branchResult.getTestCaseInfo().add(test);
                                }
                            }

                            // 3. 统计全局的覆盖信息
                            // 查看是否已经被覆盖了
                            if (branchInfoTestCaseInfoHashMap.containsKey(myBranchInfo)) {
                                continue;
                            }
                            TestCaseInfo addTestCase = null;
                            for (TestCaseInfo test : cd.getTests()) {
                                // hitsFor表示 该测试方法命中了哪些 数据(语句、分支、方法)
                                BitSet hitsFor = cd.getHitsFor(test);
                                // 检查测试方法是否覆盖了这个分支
                                BitSet branchHits = hitsFor.get(dataIndex, dataIndex + dataLength);
                                if (!branchHits.isEmpty()) {
                                    if (existTestCase.contains(test)) {
                                        branchInfoTestCaseInfoHashMap.put(myBranchInfo, test);
                                        break;
                                    } else {
                                        addTestCase = test;
                                    }
//                                    System.out.println("\t\t\t\t\tTest方法: " + test.getTestName() + " covers branch at " + branchInfo.getStartLine());
                                }
                            }
                            if (!branchInfoTestCaseInfoHashMap.containsKey(myBranchInfo) && addTestCase!=null) {
                                branchInfoTestCaseInfoHashMap.put(myBranchInfo, addTestCase);
                                existTestCase.add(addTestCase);
                            }
                        }
                    }
                }
            }
        }
    }
}
