package com.frk.mycode;

/**
 * @date 2024/5/17
 */
import soot.*;
import soot.options.Options;
import soot.toolkits.graph.*;
import soot.toolkits.scalar.*;

import java.util.*;

public class SootExample {
    public static void main(String[] args) {
        // 设置 Soot 的类路径
        Options.v().set_prepend_classpath(true);
        Options.v().set_soot_classpath(Scene.v().defaultClassPath());
        Options.v().set_src_prec(Options.src_prec_java);
        Options.v().set_output_format(Options.output_format_jimple);

        // 加载要分析的类
        SootClass sClass = Scene.v().loadClassAndSupport("com.frk.mycode.SootExample");
        Scene.v().loadNecessaryClasses();

        // 获取 main 方法
        SootMethod method = sClass.getMethodByName("main");

        // 生成 Jimple 中间表示
        Body body = method.retrieveActiveBody();

        // 构建控制流图
        UnitGraph graph = new ExceptionalUnitGraph(body);

        // 进行数据流分析
        SimpleLocalDefs localDefs = new SimpleLocalDefs(graph);

        // 遍历每个语句
        for (Unit unit : graph) {
            System.out.println("Statement: " + unit);

            // 获取语句的定义
            List<ValueBox> useBoxes = unit.getUseBoxes();
            for (ValueBox vb : useBoxes) {
                Value v = vb.getValue();
                if (v instanceof Local) {
                    List<Unit> defs = localDefs.getDefsOfAt((Local) v, unit);
                    for (Unit def : defs) {
                        System.out.println("  Depends on: " + def);
                    }
                }
            }
        }
    }
}

