package com.frk.mycode.branch;

import com.atlassian.clover.api.registry.*;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.Objects;

/**
 * @date 2024/4/12
 */
public class MyBranchInfo {
    public BranchInfo branchInfo;
    public FileInfo fileInfo;
    public String packagePath;
    public String filePath;
    public String methodName;
    public String startStatement;
    public String endStatement;
    public int branchStartLine;
    public int branchStartColumn;

    public int branchEndLine;
    public int branchEndColumn;
    public ContextSet contextSet;

    public MyBranchInfo(PackageInfo packageInfo, FileInfo fileInfo, MethodInfo methodInfo, BranchInfo branchInfo, String path) {
        this.packagePath = packageInfo.getName().replace(".", File.separator);
        this.filePath = packagePath + File.separator + fileInfo.getName();
        this.branchInfo = branchInfo;
        this.methodName = methodInfo.getName();
        this.branchStartLine = this.branchInfo.getStartLine();
        this.branchStartColumn = this.branchInfo.getStartColumn();
        this.branchEndLine = this.branchInfo.getEndLine();
        this.branchEndColumn = this.branchInfo.getEndColumn();
        this.fileInfo = fileInfo;
        this.contextSet = branchInfo.getContext();
        parseStatement(path);
    }

    private void parseStatement(String path) {
        int startLine = branchInfo.getStartLine();
        int endLine = branchInfo.getEndLine();
//        System.out.println(startLine);
//        System.out.println(endLine);
        File file = new File(path);
        file = file.getParentFile().getParentFile().getParentFile();
        // 读取源代码文件
        String sourceFilePath = Paths.get(file.getAbsolutePath() + "\\src\\main\\java", fileInfo.getContainingPackage().getName().replace(".", "/"), fileInfo.getName()).toString();
//        System.out.println(sourceFilePath);

        try (BufferedReader reader = new BufferedReader(new FileReader(sourceFilePath))) {
            int lineNumber = 0;
            String line;
            while ((line = reader.readLine()) != null) {
                lineNumber++;
                if (lineNumber == startLine) {
                    this.startStatement = line;
                } else if(lineNumber == endLine) {
                    this.endStatement = line;
                    break;
                }

            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    @Override
    public String toString() {
        return "MyBranchInfo{" +
                "filePath='" + filePath + '\'' +
                ", methodName='" + methodName + '\'' +
                ", startLine='" + branchStartLine + '\'' +
                ", contextSet=" + contextSet +
                '}';
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof MyBranchInfo)) {
            return false;
        }
        return filePath.equals(((MyBranchInfo) obj).filePath) &&
                Objects.equals(methodName, ((MyBranchInfo) obj).methodName) &&
                Objects.equals(startStatement, ((MyBranchInfo) obj).startStatement) &&
                Objects.equals(endStatement, ((MyBranchInfo) obj).endStatement) &&
                Objects.equals(branchStartLine, ((MyBranchInfo) obj).branchStartLine);
//        return filePath.equals(((MyBranchInfo) obj).filePath) &&
//                Objects.equals(methodName, ((MyBranchInfo) obj).methodName) &&
//                Objects.equals(contextSet, ((MyBranchInfo) obj).contextSet);
    }

    @Override
    public int hashCode() {
        return Objects.hash(filePath, methodName, startStatement, endStatement);
//        return Objects.hash(filePath, methodName);
    }

}
